/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.nio.ByteBuffer;
/**
 * <code>ThincCMsgEventMouseMove</code> handles the mouse moved event from canvas. 
 * it simply implements <code>MouseMotionListener</code> interfaces provided from java, and it is added
 * as a MouseMotionListener on ThincClientCanvas.
 * 
 * @author Young Jin Yoon
 *
 */
public final class ThincCMsgEventMouseMove extends ThincCMsgEvent implements MouseMotionListener{
	public static final int MESSAGE_TYPE = ThincProto.T_EV_MOTION;
	public static final int MESSAGE_SIZE = ThincProto.T_EV_MOTION_SIZE;
	public static final int TIME_QUANTUM = 100;
	private short x;
	private short y;
	
    /**
     * creates ThincCMsgEventMove object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgEventMouseMove (ThincMsgHandler h, ThincClientCanvas c)
	{
		super(h,c);
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

	/**
     * putMessage()
     * simply puts new coordination information for cursor.
     * @see ThincCMsg.putMessage()
     */
	@Override
	public void putMessage(ByteBuffer b) {
		b.putShort(x);
		b.putShort(y);
		
	}
	
	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}

	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on MouseMove",e);
		}
	}

	/**
	 * mouseDragged()
	 * 
	 * get the current position of cursor by using getX() and getY from
	 * MouseEvent object. then resize it if required. finally, it invokes
	 * process() method.
	 */
	public void mouseDragged(MouseEvent moved) {
		x = (short)moved.getX();
		y = (short)moved.getY();
		canvas.drawThincCursor(x, y);

		if(ThincSMsgFlagResize.isResized()) {
			x = ThincSMsgFlagResize.getOriginalX(x);
			y = ThincSMsgFlagResize.getOriginalY(y);
		}
		try {
			process();
		} catch (ThincException e) {
			System.err.println("process: send on MouseMove");
			e.printStackTrace();
			//TODO fill it in. make setLogin(String reason) in ThincClientCanvas!
		}
	}

	/**
	 * mouseMoved()
	 * 
	 * get the current position of cursor by using getX() and getY from
	 * MouseEvent object. then resize it if required. finally, it invokes
	 * process() method. it is exactly as same as mouseDragged() method.
	 */
	public void mouseMoved(MouseEvent moved) {
		x = (short)moved.getX();
		y = (short)moved.getY();
		canvas.drawThincCursor(x, y);

		if(ThincSMsgFlagResize.isResized()) {
			x = ThincSMsgFlagResize.getOriginalX(x);
			y = ThincSMsgFlagResize.getOriginalY(y);
		}
		try {
		
			process();
		} catch (ThincException e) {
			System.err.println("process: send on MouseMove");
			e.printStackTrace();
			//TODO fill it in. make setLogin(String reason) in ThincClientCanvas!
		}
	}
}
