/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * let the server knows that client supports no video.
 * @author  Young Jin Yoon
 * @see  ThincCMsgInitRequest
 */
public final class ThincCMsgInitRequestNoVideo extends ThincCMsgInitRequest {

	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_NOVIDEO; 
	private ThincMsgInitReplyHandler initHandler;
	
	/**
     * creates ThincCMsgInitRequestNoVideo object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgInitRequestNoVideo(ThincMsgHandler h, ThincClientCanvas c, ThincMsgInitReplyHandler init) {
		super(h, c);
		initHandler = init;
	}
	
	/**
	 * putMessage()
	 * 
	 * puts request type and 0 for unused field, respectively.
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		b.putShort((short)INIT_MESSAGE_TYPE);
		b.putShort((short)0);
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}

	@Override
	public void process() throws ThincException {
		
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on NoVideo",e);
		}
		/* does not send reply from server. so send next message right after this. */
		initHandler.sendNextMessage(); 
	}

}
