/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * replies the capability for security to server.
 * @author  Young Jin Yoon
 * @see  ThincCMsgInit
 */
public final class ThincCMsgInitSecCaps extends ThincCMsgInit {
	public static final int MESSAGE_TYPE = ThincProto2.T_CLIENT_SEC_CAPS;
	public static final int MESSAGE_SIZE = ThincProto2.T_CLIENT_SEC_CAPS_SIZE;

	private ThincClient client;
	
	/**
     * creates ThincCMsgInitRequestVideoClientFormat object.
     * @param tc <code>ThincClient</code> to get the security capability
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgInitSecCaps(ThincClient tc, ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
		client = tc;
}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

	/**
	 * putMessage()
	 * 
	 * puts security capability flags only.
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		int sec = client.getSecCap();
		b.putInt(sec);
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}

	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException ("process: Error in handler.send",e);
		}
	}

}
