/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.Rectangle;
import java.nio.ByteBuffer;
/**
 * <code>ThincCMsgEventMouseButton</code> is for sending the message that the region
 * in frame buffer should be refreshed.
 * In my implementation, it simply request the entire resized frame buffer.
 * 
 * @author Young Jin Yoon
 * @see ThincCMsgResize
 */
public final class ThincCMsgResizeUpdate extends ThincCMsgResize {
	public static final int MESSAGE_TYPE = ThincProtoResize.T_RESZ_UPDATE;
	public static final int MESSAGE_SIZE = ThincProtoResize.T_RESZ_UPDATE_SIZE;

	public static final int DATA_SIZE = ThincProto.T_RECTREGION_SIZE;
	
	private int x;
	private int y;
	private int width;
	private int height;

    /**
     * creates ThincCMsgEventMouseButton object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincCMsgResizeUpdate(ThincMsgHandler h, ThincClientCanvas c,int x, int y, int width, int height) {
		super(h, c);
		this.width = width;
		this.height = height;
		this.x = x;
		this.y = y;
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}


	/**
     * putMessage()
     * puts the number of region as 1. then 0,0,its width,its height to refresh
     * entire screen.
     * 
     * @see ThincCMsg.putMessage()
     */
	@Override
	public void putMessage(ByteBuffer b) {
		b.putInt(1);			// one region to refresh
		b.putShort((short) x);	// x coord
		b.putShort((short) y);	// y coord
		b.putShort((short)width); // entire width for canvas
		b.putShort((short)height); // entire height for canvas
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE + DATA_SIZE;
	}

	public short headerSize() {
		return MESSAGE_SIZE;
	}

	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on ResizeEvent",e);
		}
		
	}

}
