/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDesktopPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
/**
 * <code>ThincFrame</code> is the frame of the <code>ThincClientCanvas</code>. it also acts as login window before drawing actual image from the server.
 * @author  Young Jin Yoon
 */
public class ThincFrame extends javax.swing.JFrame implements MouseListener {

	/**
	 * serialVersionUID generated from compiler
	 */
	private static final long serialVersionUID = -7788045202680658080L;

	/* pointer to the object */
	private ThincClient client;
	private ThincClientCanvas canvas;
	
	/* various UI to get the input from user */
	private JDesktopPane ThincDesktopPane; 	// main panel
	private JLabel Server; 					//letter "Server"
	private JTextField ServerName; 			// server text field for user input 
	private JCheckBox Sound; 				// check box enabling / disabling Sound;
	private JButton Login; 					// button "Login"
	private JPasswordField PasswdName;		// password field for user input
	private JLabel PASSWD;					// letter "PASSWD"
	private JLabel ID;						// letter "ID"
	private JTextField IDName;				// id text field for user input
	private JLabel Msg;						// for printing error message from the network
	private JLabel ThincLogo;				// ThincClient Logo
	private JCheckBox Authentication;		// check box endabling / disabling auth
	private JTextField PortNum;				// port number text field for user input
	private JTextField SndPortNum;			// sound port number text field for user input
	private JLabel SndPort;					// letter "Sound Port"
	private JCheckBox SSL;					// check box enabling / disabling SSL
	private JLabel Port;					// letter "Port"
	/**
	 * creates <code>ThincFrame</code> object. it sets User interfaces
	 * using initGUI() method and set the WindowListener for closing
	 * window.
	 * @param cl ThincClient object which created this
	 * @throws ThincException
	 */
	public ThincFrame(ThincClient cl) throws ThincException{
		super();
		initGUI();
		client = cl;
		addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
               try {
            	   client.close();
               } catch(Exception e1) {
            	   e1.printStackTrace();
               }
                System.exit(0);
            }
        });	
	}
	/**
	 * setCanvas() disables the main login components and registers the <code>ThincClientCanvas</code> into this object. it also set <code>ThincClientCanvas</code> object onto the ThincFrame.
	 * @param ca  <code>ThincClientCanvas</code> object to set
	 */
	public void setCanvas(ThincClientCanvas ca) throws ThincException{
		canvas = ca;
		
		getContentPane().remove(ThincDesktopPane); // just remove login main panel from window

		/* set the size of ThincClientCanvas object and add it into window */
		getContentPane().add(canvas, BorderLayout.CENTER); 
		canvas.setBounds(0,0, getSize().width, getSize().height); 
		canvas.repaint();
		canvas.setVisible(true);
		repaint();
	}

	/**
	 * setLogin()
	 *
	 * sets the login window again. it removes the <code>ThincClientCanvas</code> object
	 * from the window, then add login main panel into window.
	 */
	public void setLogin() {
		getContentPane().remove(canvas); // remove canvas
		getContentPane().add(ThincDesktopPane); // add login main panel
		repaint();
	}
	
	/**
	 * initGUI()
	 * 
	 * initializes the login GUI for user.
	 */
	private void initGUI() throws ThincException{
		try {
			setTitle("ThincClient"); // title : "ThincCLient"
			{
				/* create desktop panel */
				ThincDesktopPane = new JDesktopPane(); 
				/* add it into window */
				getContentPane().add(ThincDesktopPane, BorderLayout.CENTER);
				
				/* Server setting */
				{
					Server = new JLabel();
					Server.setText("Server");
					Server.setBounds(14, 21, 63, 28);
					ThincDesktopPane.add(Server);
				}
				
				/* Port setting */
				{
					Port = new JLabel();
					Port.setText("Port");
					Port.setBounds(294, 21, 49, 28);
					ThincDesktopPane.add(Port);
				}
				/* SndPort setting */
				{
					SndPort = new JLabel();
					SndPort.setText("Sound Port");
					SndPort.setBounds(255, 51, 100, 28);
					ThincDesktopPane.add(SndPort);
				}
				/* ServerName setting */
				{
					 ServerName = new JTextField();
					 ServerName.setText(ThincClient.DEFAULT_HOST); // default host
					ServerName.setBounds(84, 21, 196, 28);
					ThincDesktopPane.add(ServerName);
				}

				/* PortNum setting */
				{
					 PortNum = new JTextField();
					 PortNum.setText(Integer.toString(ThincClient.DEFAULT_PORT)); // default port
					PortNum.setBounds(343, 21, 105, 28);
					ThincDesktopPane.add(PortNum);
				}
				/* SndPortNum setting */
				{
					SndPortNum = new JTextField();
					SndPortNum.setText(Integer.toString(ThincClient.DEFAULT_PORT+ ThincClient.OFFSET_SND_PORT)); // default port
					SndPortNum.setBounds(343, 51, 105, 28);
					ThincDesktopPane.add(SndPortNum);
				}

				/* Authentication setting */
				{
					Authentication = new JCheckBox();
					Authentication.setText("Authentication");
					Authentication.setBounds(84, 117, 140, 21);
					Authentication.setBorderPaintedFlat(true);
					Authentication.setSelected(true);
					/* set id and passwd as editable if auth is enabled,
					   not editable if it is disabled. */
					Authentication.addChangeListener(new ChangeListener() {
					      public void stateChanged(ChangeEvent changeEvent) {
					    	  if(!Authentication.isSelected()) {
					    		  IDName.setEditable(false);
					    		  PasswdName.setEditable(false);
					    	  } else {
					    		  IDName.setEditable(true);
					    		  PasswdName.setEditable(true);
					    	  }
					        }
					      });
					ThincDesktopPane.add(Authentication);
				}
				/* IDName setting */
				{
					IDName = new JTextField();
					IDName.setText("yy2223"); // default ID
					IDName.setBounds(84, 147, 147, 28);
					ThincDesktopPane.add(IDName);
				}
				
				/* ID setting */
				{
					ID = new JLabel();
					ID.setText("USER");
					ID.setBounds(14, 147, 56, 28);
					ThincDesktopPane.add(ID);
				}
				
				/* PASSWD setting */
				{
					PASSWD = new JLabel();
					PASSWD.setText("PASS");
					PASSWD.setBounds(14, 177, 56, 28);
					ThincDesktopPane.add(PASSWD);
				}
				
				/* PasswdName setting */
				{
					PasswdName = new JPasswordField();
					PasswdName.setBounds(84, 177, 147, 28);
					ThincDesktopPane.add(PasswdName);
				}
				/* Login setting */
				{
					Login = new JButton();
					Login.setText("Login");
					Login.setBounds(315, 147, 98, 56);
					Login.addMouseListener(this); //see mouseClicked() method
					ThincDesktopPane.add(Login);
				}
				/* Sound setting */
				{
					Sound = new JCheckBox();
					Sound.setText("Sound");
					Sound.setBounds(84, 57, 98, 21);
					Sound.setSelected(true);
					Sound.addChangeListener(new ChangeListener() {
					      public void stateChanged(ChangeEvent changeEvent) {
					    	  if(!Sound.isSelected()) {
					    		  SndPortNum.setEditable(false);
					    	  } else {
					    		  SndPortNum.setEditable(true);
					    	  }
					        }
					      });
					ThincDesktopPane.add(Sound);
				}
				/* SSL setting */
				{
					SSL = new JCheckBox();
					SSL.setText("SSL");
					SSL.setBounds(84, 87, 98, 21);
					SSL.setSelected(true);
					ThincDesktopPane.add(SSL);
				}
				/* ThincLogo setting */
				{
					ThincLogo = new JLabel();
					ThincLogo.setText("Thinc Client (JAVA)");
					ThincLogo.setBounds(77, 237, 329, 42);
					ThincDesktopPane.add(ThincLogo);
				}

				/* Msg setting */
				{
					Msg = new JLabel();
					Msg.setBounds(77, 217, 336, 28);
					ThincDesktopPane.add(Msg);
				}
			}
			/* set frame size and visibility */
			pack();
			setSize(480,400);
			setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
			throw new ThincException("initGUI: login GUI setting failed",e);
		}
	}

	/**
	 * getMsgLabel()
	 * 
	 * returns the <code>JLabel</code> object to print the error message onto user.
	 * @return <code>JLabel</code> object to print the error message
	 */
	public JLabel getMsgLabel() { return Msg; }
	
	
	/* implemented methods from MouseListener interfaces below*/

	/**
	 * mouseClicked()
	 * 
	 * if mouse i\s clicked on login button, it is invoked by <code>MouseListener</code>.
	 * it gets the all information for connecting server and put it into
	 * <code>ThincClient</code> object using setLoginProperty() methods in <code>ThincClient</code>
	 * class
	 */
	public void mouseClicked(MouseEvent arg0) {
		String server,passwd,id;
		int port, sndPort;
		boolean auth, sound, ssl;
		
		/* get server name, password, id, port, ssl and auth information */
		server = ServerName.getText();
		passwd = String.copyValueOf(PasswdName.getPassword());
		id = IDName.getText();
		port = Integer.parseInt(PortNum.getText());
		sound = Sound.isSelected();
		sndPort = Integer.parseInt(SndPortNum.getText());
		ssl = SSL.isSelected();
		auth = Authentication.isSelected();

		/* put the information into ThincClient class using setLoginProperty method */
		client.setLoginProperty(server,port,auth,ssl,sound,sndPort,id,passwd);
	}
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mousePressed(MouseEvent arg0) {}
	public void mouseReleased(MouseEvent arg0) {}
}
