/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * handles changing the color of the cursor. it comes with new foreground and
 * background colors.
 * @author Young Jin Yoon
 * @see ThincSMsgCursor
 */
public final class ThincSMsgCursorColor extends ThincSMsgCursor {
	public static final int MESSAGE_TYPE = ThincProto.T_CURSOR_COLOR;
	private static final int MESSAGE_SIZE = ThincProto.T_CURSOR_COLOR_SIZE;

	/**
     * creates ThincSMsgCursorColor object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgCursorColor(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * get the foreground and background color for cursor and change the color using
	 * colorChangeCursor() method in ThincClientCanvas class.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		int fg,bg;
		/* get header message and parse it */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("process: error on Cursor Change",e);
		}
		fg = bb.get(); //get the foreground color
		bg = bb.get(); //get the background color

		//System.out.println("ThincSMsgCursorColor: fg-"+Integer.toBinaryString(fg)+",bg-"+Integer.toBinaryString(bg));

		fg = canvas.argbToAbgr(fg) | 0xff000000;
		bg = canvas.argbToAbgr(bg) | 0xff000000;

		canvas.colorChangeCursor(fg, bg); // change the color of cursor
	}



}
