/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * handles move the cursor to new coordination. it comes with new x and y coordinates.
 * now it only receive and parse the message but not moving cursor around. To move the
 * cursor, delete '//' in the last line of this class, which 
 * actually move the cursor.
 * @author Young Jin Yoon
 * @see ThincSMsgCursor
 */
public final class ThincSMsgCursorMove extends ThincSMsgCursor {
	public static final int MESSAGE_TYPE = ThincProto.T_CURSOR_MOVE;
	public static final int MESSAGE_SIZE = ThincProto.T_CURSOR_MOVE_SIZE;

	/**
     * creates ThincSMsgCursorMove object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgCursorMove(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * get the new coordination for cursor and change the position using
	 * moveThincCursor() method in ThincClientCanvas class.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb = null;
		short x,y;
		/* get header message and parse it */		
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("process: error on Cursor Change",e);
		}
		/* get the new coordinate for cursor */	
		x = bb.getShort();
		y = bb.getShort();
		if(ThincSMsgFlagResize.isResized()) {
			x = ThincSMsgFlagResize.getResizedX(x);
			y = ThincSMsgFlagResize.getResizedY(y);
		}
		/* converting short with unsigned int and invokes moveThincCursor in ThincClientCanvas class */
		/* TODO: because the server side does not prepared to move the cursor properly,
		 *       It is commented out. it should works very well ;) */
		// canvas.moveThincCursor((int)(x & 0xFFFF), (int)(y & 0xFFFF)); 
	}

}
