/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * handles show/hide the cursor. it comes with a flag which represents show / hide
 * the cursor.
 * @author Young Jin Yoon
 * @see ThincSMsgCursor
 */
public final class ThincSMsgCursorShowHide extends ThincSMsgCursor {
	public static final int MESSAGE_TYPE = ThincProto.T_CURSOR_SHOWHIDE;
	public static final int MESSAGE_SIZE = ThincProto.T_CURSOR_SHOWHIDE_SIZE;
	
	public static final int SHOW = ThincProto.T_SHOW; // flag for show
	public static final int HIDE = ThincProto.T_HIDE; // flag for hide
	
	public ThincSMsgCursorShowHide(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}
	
	/**
	 * process()
	 * 
	 * get the flag for showing cursor and change it into show/hide using
	 * showThincCursor() method in ThincClientCanvas class.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
	
		ByteBuffer bb = null;
		byte showhide;
		/* get header message and parse it */		
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("process: error on Cursor Change",e);
		}
		showhide = bb.get(); //flag to show/hide
		
		/* show/hide cursor using showThincCursor method in ThincClientCanvas */
		if(showhide == SHOW) canvas.showThincCursor(true);
		else canvas.showThincCursor(false);
	}
}
