/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
import java.nio.IntBuffer;

/**
 * class for receiving init images for frame buffer.
 * @author  Young Jin Yoon
 * @see  ThincSMsgInitReply
 */
public final class ThincSMsgInitReplyFBData extends ThincSMsgInitReply {
	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_FBDATA; 
	public static final int MESSAGE_SIZE = ThincProto2.T_REPLY_FBDATA_SIZE;
	private ThincSMsgFlagFBZData zdata;

	/**
     * creates ThincSMsgInitReplyFBData object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgInitReplyFBData(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
		zdata = new ThincSMsgFlagFBZData((ThincMsgHandler)handler);
	}

	/**
	 * process()
	 * 
	 * gets the message containing information for init frame buffer image, then
	 * receiving the init frame buffer image and draw it into frame buffer by using
	 * setFrameBuffer() and drawFrameBuffer methods in <code>ThincClientCanvas</code>
	 * class.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		
		ByteBuffer bb;
		byte[] unziped;
		int flags;
		IntBuffer unzipedIntBuf;
		int[] unzipedInt;
		/* get the information for init frame buffer */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: error on FBData",e);
		}
		flags = bb.getInt(); // flags for compression
			
		if(flags != 0){// if it is compressed,
			for(int i=0 ; i < canvas.getFBHeight(); i++) {
				unzipedIntBuf = zdata.process(); //received each width with ziped image, process it.
				unzipedInt = new int[unzipedIntBuf.limit()];
				unzipedIntBuf.get(unzipedInt);
				canvas.setFrameBuffer(unzipedInt);//set each width image into frame buffer.
				unzipedIntBuf.clear();
			}
		} else { //if it is unziped
			unziped = new byte [((ThincMsgHandler)handler).getLength()-MESSAGE_SIZE];
			try {
				bb = handler.getMessage(((ThincMsgHandler)handler).getLength()-MESSAGE_SIZE);
			} catch (ThincException e) {
				throw new ThincException("process: error on Contents of FBData",e);
			}
			bb.get(unziped);
			unzipedIntBuf = bb.asIntBuffer();
			unzipedInt = new int[unzipedIntBuf.limit()];
			unzipedIntBuf.get(unzipedInt);
			canvas.setFrameBuffer(unzipedInt); 
				// set one big image into frame buffer. because it is not tested yet, I cannot
				// promise it works. if it does not, try to modify this else clause
				// with for loop just like if clause above. 
				//( from this line : for(int i=0 ; i < canvas.getFBHeight(); i++) )
		}
		/* drawFrameBuffer which is already set by setFrameBuffer() method. */
		canvas.drawFrameBuffer();
	}


}
