/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for receiving frame buffer information.
 * 
 * @author Young Jin Yoon
 * @see ThincSMsgInitReply
 */
public final class ThincSMsgInitReplyFBInfo extends ThincSMsgInitReply {
	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_FBINFO;
	public static final int MESSAGE_SIZE = ThincProto2.T_REPLY_FBINFO_SIZE;

	/**
     * creates ThincSMsgInitReplyFBInfo object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */	
	public ThincSMsgInitReplyFBInfo(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}

	/**
	 * process()
	 * 
	 * gets the message containing information for frame buffer, then set it into
	 * <code> ThincClientCanvas </code> class by using createFrameBuffer method.
 	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		byte depth;
		byte bpp;
		short width,height;
		/*get the frame buffer information message from the network */
		try {
		bb = handler.getMessage(MESSAGE_SIZE);
		}catch (ThincException e) {
			throw new ThincException("process: Error on getMessage in FBInfo",e);
		}
		bb.getShort();
		depth = bb.get(); //depth for each pixel
		bpp = bb.get(); // bit per pixel in frame buffer
		width = bb.getShort(); // width for frame buffer
		height = bb.getShort(); // height for frame buffer
		
		/* create frame buffer using createFrameBuffer in ThincClientCanvas class */
		canvas.createFrameBuffer(width, height, bpp, depth);
	}

}
