/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for receiving security capability from the server. it also send the 
 * <code> ThincCMsgInitSecCaps </code> to notify the client supported security capability.
 * @author Young Jin Yoon
 * @see ThincSMsgInit
 */
public final class ThincSMsgInitSecCaps extends ThincSMsgInit {
	public static final int MESSAGE_TYPE = ThincProto2.T_SERV_SEC_CAPS;
	public static final int MESSAGE_SIZE = ThincProto2.T_SERV_SEC_CAPS_SIZE;
	
	/**
     * creates ThincSMsgInitSecCaps object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */	
	public ThincSMsgInitSecCaps(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}

	/**
	 * process()
	 * 
	 * receive security capability from the server and use bitwise & operator
	 * to get client security capability correspond with server. finally, 
	 * it creates and sends <code> ThincCMsgInitRequestClientFormat </code> 
	 * to notify the client supported video format.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException{
		int clientSec = 0;
		int servSec;
		ThincCMsgInitSecCaps cmsg;
		ByteBuffer bb = null;
		/* receive the security capability from the network */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgInitSecCaps, receiving security capability failed",e);
		}
		servSec = bb.getInt(); // server security capability
		clientSec = client.getSecCap(); // client security capability
		client.setSecCap(clientSec & servSec);  // set client security capability with
												// server security capability.

		/* create and send ThincCMsgInitSecCaps */
		cmsg = new ThincCMsgInitSecCaps(client,(ThincMsgHandler)handler, canvas);
		try {
			cmsg.process();
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgInitSecCaps, sending security capability failed",e);
		}
	}
}
