/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;

/**
 * class for getting the information for configuration of sound devices.
 * @author Young Jin Yoon
 * @see ThincSMsgSound
 */

public final class ThincSMsgSoundOpen extends ThincSMsgSound {
	public static final int MESSAGE_TYPE = ThincProtoSound.T_SND_OPEN;
	public static final int MESSAGE_SIZE = ThincProtoSound.T_SND_OPEN_SIZE;
	/**
     * creates ThincSMsgSoundOpen object.
     * @param h <code>ThincSoundHandler</code> to handle the message
 	 */
	public ThincSMsgSoundOpen(ThincSoundHandler h) {
		super(h);
	}
	
	/**
	 * process()
	 * 
	 * gets the rate, bits, channels and endian for sound device and set it into 
	 * <code>AudioFormat</code> and <code>DataLine.Info</code> object.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException{
		ByteBuffer bb;
		short rate;
		byte bits, channels, endian;
		boolean isEndian;
		/* receive  rate, bits, channels and endian */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgSoundOpen");
		}
		bb.get();
		bits = bb.get(); // bits for sound
		rate = bb.getShort(); // rate for sound
		channels = bb.get(); // channels for sound
		endian = bb.get(); // big/small endian for sound
		if(endian == 1) isEndian = false;
		else if(endian == 0) isEndian = true;
		else isEndian = true;
//		else throw new ThincException("process: in ThincSMsgSoundOpen, invalid endian"+endian);
		/* set the information from the network into AudioFormat and DataLine.Info object */
		format = new AudioFormat((int)(rate & 0xffff),(int)(bits & 0xff),(int)(channels & 0xff),true,isEndian);
        playInfo = new DataLine.Info(SourceDataLine.class, format);		
	}
}
