/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for moving the video stream's location.
 * @author Young Jin Yoon
 * @see ThincSMsgVideo
 */
public final class ThincSMsgVideoMove extends ThincSMsgVideo {
	public static final int MESSAGE_TYPE = ThincProtoVideo.T_VIDEO_MOVE;
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_MOVE_SIZE;

	/**
     * creates ThincSMsgVideoMove object.
     * @param h <code>ThincSoundHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
 	 */
	public ThincSMsgVideoMove(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * gets the stream id and new coordination from the server to move, 
	 * then moves the stream by modifying stream information from the vector.
	 * 
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException{
		ByteBuffer bb;
		ThincVideoStream ts;
		int id;
		short x, y;
		/* get the stream id and new coordination from the network */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgVideoNext");
		}
		id = bb.getInt(); //stream id
		x = bb.getShort(); //new x for stream
		y = bb.getShort(); //new y for stream
		
		/* get a stream information object from the vector and modify the
		 * x and y in the stream information. */
		ts = (ThincVideoStream) streamId.get(id);
		ts.x = x;
		ts.y = y;
	}

}
