/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
import java.nio.IntBuffer;

/**
 * class for playing the video stream.
 * @author Young Jin Yoon
 * @see ThincSMsgVideo
 */
public final class ThincSMsgVideoNext extends ThincSMsgVideo {
	public static final int MESSAGE_TYPE = ThincProtoVideo.T_VIDEO_NEXT;
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_NEXT_SIZE;
	
	/**
     * creates ThincSMsgVideoNext object.
     * @param h <code>ThincSoundHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
 	 */
	public ThincSMsgVideoNext(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * gets the stream id,size and timestamp from the server, 
	 * then show the frame onto the canvas by using the generated 
	 * ABGR image from <code>ThincVideoFormat</code>.
	 * 
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException{
		int id, size, timestamp;
		int x,y,dst_width,dst_height;
		ByteBuffer bb;
		IntBuffer image;
		/* gets the stream id,size and timestamp from the network */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgVideoNext",e);
		}
		id = bb.getInt(); //stream id
		size = bb.getInt(); // frame size
		timestamp = bb.getInt(); // timestamp for this frame
		
		/* get the stream information from the vector */
		ThincVideoStream src = (ThincVideoStream) streamId.get(id);
		
		/* calculating x,y,width and height if the ThincClientCanvas is resized */
		x = (ThincSMsgFlagResize.isResized())?ThincSMsgFlagResize.getResizedX((short) src.x):src.x;
		y = (ThincSMsgFlagResize.isResized())?ThincSMsgFlagResize.getResizedY((short) src.y):src.y;
		dst_width = (ThincSMsgFlagResize.isResized())?ThincSMsgFlagResize.getResizedWidth((short) src.dst_width):src.dst_width;
		dst_height = (ThincSMsgFlagResize.isResized())?ThincSMsgFlagResize.getResizedHeight((short) src.dst_height):src.dst_height;

		/* convert YUV frame image into ABGR image */
		try {
			image = src.format.process(src.width,src.height,size);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgVideoNext",e);
		}
		
		/* if it is first frame for stream, set the timestamp and draw */
		if(!src.firstFlag) {
			src.timestamp = timestamp;
			if(image != null) canvas.drawImageScaled(image, x, y, src.width, src.height, dst_width, dst_height,ThincClientCanvas.TYPE_VIDEO);
			src.firstFlag = true;
			return;
		}
		/* if it is valid frame (incoming timestamp should be larger then latest timestamp)
		 * for stream, set the timestamp and draw */
//		if(image != null && (long) (src.timestamp & 0xffffffff)< (long)(timestamp& 0xffffffff)) {
			canvas.drawImageScaled(image, x, y, src.width, src.height, dst_width, dst_height,ThincClientCanvas.TYPE_VIDEO);
			src.timestamp = timestamp;
//		}
//		else System.err.println("Discarded - src.timestamp:"+src.timestamp+" timestamp:"+timestamp);
	}
}
