/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
import java.nio.IntBuffer;


/**
 * class for YUY2 video format used in <code>ThincSMsgVideo</code>. 
 * @author  Young Jin Yoon
 * @see ThincVideoFormat
 * @see  http://www.fourcc.org
 */
public class ThincVideoFormatYUY2 extends ThincVideoFormat {
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_INFO_YUY2_SIZE;
	int pitch;
	ThincMsgHandler handler;
	/**
     * creates ThincVideoFormatYUY2 object. it also receives the
     * message format information from the server.
     * @param th <code>ThincMsgHandler</code> to handle the message
	 * @param height 
	 * @param width 
 	 */
	public ThincVideoFormatYUY2(ThincMsgHandler th, short width, short height) throws ThincException{
		ByteBuffer bb;
		handler = th;
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("ThincVideoFormatYUY2: cannot receive message format",e);
		}
		pitch = bb.getInt(); // pitch for YUY2
	}

	@Override
	public IntBuffer process(int width, int height, int size) throws ThincException{
		ByteBuffer bb;
		IntBuffer retVal = IntBuffer.allocate(width * height);
		int rgb0, rgb1, yuv;
		byte u0, y0, v0, y1;
		/* get all YUV images into ByteBuffer */
		try {
			bb = handler.getMessage(size);
		} catch (ThincException e) {
			throw new ThincException ("ThincVideoFormatYUY2: cannot receive message format",e);
		}		
		/* try to set yuv for each pixel and converts it into ARGB pixel */
		for(int i=0; i<size/4; i++) {
			yuv = bb.getInt();
			y0 = (byte) (yuv >> 24);
			u0 = (byte)	(yuv >> 16);
			y1 = (byte)	(yuv >> 8);
			v0 = (byte)	yuv;
			rgb0 = yuvToRgb(y0,u0,v0);
			rgb1 = yuvToRgb(y1,u0,v0);
			/*if pitch is 240 and width is 480, it should draws 
			 * the same two pixel on the neighbour */
			for(int j=0; j< (int)(width / pitch); j++) 
				retVal.put(rgb0);
			for(int j=0; j< (int)(width / pitch); j++) 
				retVal.put(rgb1);			
		}
		return retVal;
	}
}
