/* Copyright 2005, 2007 Ricardo Baratto */
/* Copyright 2005, 2007 Columbia University */
/*
 * Header for instrumentation code
 *
 * Author:
 * Ricardo Baratto
 * Network Computing Lab - Columbia University
 */

/* set tabstop = 8 */

#ifndef _TC_INSTRUMENT_H_
#define _TC_INSTRUMENT_H_

#if defined(TC_INSTRUMENT) || defined(TC_TIMELINE)
#include <asm/msr.h>
#endif

#define gettime_ms(t) { \
	struct timeval tv;		\
	gettimeofday(&tv, NULL);	\
	t = (tv.tv_sec*1000) + (tv.tv_usec/1000); \
}

#define gettime_us(t) do { \
	struct timeval __tv;        \
		gettimeofday(&__tv, NULL);  \
		t = (__tv.tv_sec*1000000) + (__tv.tv_usec); \
} while(0)

/* basic stats */
#ifdef TC_STATS
struct tc_stats {
	/* received */
	unsigned int rawcount;
	unsigned int rawbytes;
	unsigned int copycount;
	unsigned int copybytes;
	unsigned int sfillcount;
	unsigned int sfillbytes;
	unsigned int pfillcount;
	unsigned int pfillbytes;
	unsigned int glyphcount;
	unsigned int glyphbytes;
	unsigned int bilevelcount;
	unsigned int bilevelbytes;

	unsigned int cursorcount;
	/* sent -- unused for now (functions don't have access to this
	 * struct) */
	unsigned int motioncount;
	unsigned int buttoncount;
	unsigned int keycount;
	/* total */
	unsigned int recvcount;
	unsigned int recvbytes;
	unsigned int sentcount;
	unsigned int sentbytes;
	unsigned int soundcount;

	/* time */
	unsigned long rawtime;
	unsigned long copytime;
	unsigned long sfilltime;
	unsigned long pfilltime;
	unsigned long glyphtime;
	unsigned long bileveltime;
};
#define tc_stats_inc(a) (a)++
#define tc_stats_add(a, b)	(a) += (b)
#define tc_stats_vars(...)	unsigned long __VA_ARGS__
#define tc_stats_dbg(l, dbg)	tclient_dbg(l, dbg)
#define tc_stats_msg(...)	tclient_msg(__VA_ARGS__)
#define tc_stats_zero(a)	(a) = 0
#define tc_stats_start(s)	gettime_ms(s)
#define tc_stats_end(s, e, which)	\
	gettime_ms(e);			\
	tc_stats_add(which, (e) - (s))
#else
#define tc_stats_inc(a)
#define tc_stats_add(a, b)
#define tc_stats_vars(...)
#define tc_stats_dbg(l, dbg)
#define tc_stats_msg(...)
#define tc_stats_zero(a)
#define tc_stats_start(s)
#define tc_stats_end(s, e, which)

#endif	/* TC_STATS */

/* hardcore stats */
#ifdef TC_INSTRUMENT
long double get_cpu_speed_measure();
long double get_cpu_speed_proc();
inline long double ticks_to_sec(long double speed, 
	unsigned long long ticks);

struct tc_instrument {
	/* totals */
	unsigned long long run_time;
	unsigned long long total;
	unsigned long long handle_msg; /* time spent calling handleMsg */
	unsigned long long handle_x; /* time spent calling handleEvents */
	unsigned long long handle_snd; /* time spent calling handleSound */
	unsigned long long poll;	/* time spent sleeping in poll */

	unsigned long long total_raw;
	unsigned long long total_copy;
	unsigned long long total_sfill;
	unsigned long long total_pfill;
	unsigned long long total_glyph;
	unsigned long long total_bilevel;
	unsigned long long total_cursor;
	unsigned long long total_ping;

	/* the basics */
	unsigned long long x_evs;
	unsigned long long x_up;
	unsigned long long basic_net;

	/* x stuff */
	unsigned long long x_expose;
	unsigned long long x_update;

	/* draw */
	unsigned long long draw_raw;
	unsigned long long draw_copy;
	unsigned long long draw_sfill;
	unsigned long long draw_pfill;
	unsigned long long draw_glyph;
	unsigned long long draw_bilevel;

	/* handle events (without drawing part) */
	unsigned long long handle_raw;
	unsigned long long handle_copy;
	unsigned long long handle_sfill;
	unsigned long long handle_pfill;
	unsigned long long handle_glyph;
	unsigned long long handle_bilevel;

	/* video */
	unsigned long long video_start;
	unsigned long long video_end;
	unsigned long long video_next;
	unsigned long long video_mod;

	/* resize */
	unsigned long long do_resize;

	/* ssl or normal read/write */
	unsigned long long read;
	unsigned long long io_read;	/* the actual read call */
	unsigned long long write;
	unsigned long long io_write;	/* the actual write call */

	/* buffer */
	unsigned long long read_memcpy;
	unsigned long long read_poll;

	/* cache */
	unsigned long long cache_add;
	unsigned long long cache_get;

	/* raw */
	unsigned long long raw_comp;
	unsigned long long raw_memcpy;
	unsigned long long raw_malloc;
	unsigned long long raw_read;
	unsigned long long raw_read_nonblock;
	unsigned long long raw_addcache;
	unsigned long long raw_getcache;

	/* compression */
	unsigned long long comp_init;
	unsigned long long comp_read;
	unsigned long long comp_comp;
	unsigned long long comp_end;

	/* sfill */
	unsigned long long sfill_read;
	unsigned long long sfill_creategc;
	unsigned long long sfill_changegc;
	unsigned long long sfill_fill;
	unsigned long long sfill_addcoords;

	/* window */
	unsigned long long win_copy;

	/* sync */
	unsigned long long win_sync;
	unsigned long long pfill_sync;
	unsigned long long pfill_sync_prepare;
	unsigned long long sfill_sync_prepare_create;
	unsigned long long sfill_sync_prepare_change;
	unsigned long long sfill_sync_draw;
	unsigned long long glyph_sync;
	unsigned long long raw_sync;

	/* some counters */
	unsigned long long sfill_create_gc;
	unsigned long long sfill_change_gc;
	unsigned long long sfill_change_color;
	unsigned long long sfill_byhand;

	unsigned long long raw_pixels;
	unsigned long long sfill_pixels;
	unsigned long long pfill_pixels;
	
	/* initialization */
	/*** unused for now ***/
	unsigned long long init;
	unsigned long long read_fb;
	unsigned long long init_x;

	unsigned long long num_updates;

};

#define instr_msg(...)	tclient_msg(__VA_ARGS__)

#if 0
#define DONT_USE_MSR
#endif

#ifdef DONT_USE_MSR
#define gettime_tsc(t) gettime_ms(t)

#define instr_print(fb, speed, which) \
	tclient_msg(#which ": %Ldms\n", (fb)->instr.which)

#else	/* USE MSR */
#define gettime_tsc(t) do {\
	unsigned long __l, __h;		\
	rdtsc(__l,__h);		\
	(t) = __h;		\
	(t) <<= 32;		\
	(t) |= __l;		\
} while(0)

#define instr_print(fb, speed, which) \
	tclient_msg(#which ": %.4Lfms\n", ticks_to_sec((speed),  \
		(fb)->instr.which))

#endif	/* DONT_USE_MSR */

#define instr_add_to(fb, s, e, which) do { \
	(fb)->instr.which += (e) - (s);	\
} while(0)

#define instr_inc(fb, which) (fb)->instr.which++

#define instr_start(s) \
	gettime_tsc(s)

#define instr_end(fb, s, e, which) do {	\
	gettime_tsc(e); 		\
	instr_add_to(fb, s, e, which);	\
} while(0)

/* use to wrap instr calls around one-liner */
#define instr_call(fb, s, e, which, call) do {	\
	instr_start(s);				\
	call;					\
	instr_end(fb, s, e, which);		\
} while(0)

#define instr_init(fb, which) \
	(fb)->instr.which = 0;

/* don't you just love gcc? */
#define instr_vars(var1, ...) \
	unsigned long long var1, __VA_ARGS__

#define instr_print_simple(fb, which) \
	tclient_msg(#which ": %Ld\n", (fb)->instr.which)

#define instr_add(fb, which, a) ((fb)->instr.which += (a))

#else	/* TC_INSTRUMENT */
#define instr_msg(...)
#define instr_add_to(fb, s, e, which)
#define instr_end(fb, s, e, which)
#define instr_call(s, e, fb, which, call)	call
#define instr_start(s)
#define instr_print(fb, speed, which)
#define instr_print_simple(fb, which)
#define instr_init(fb, which)
#define instr_vars(var1, ...)
#define instr_inc(fb, which)
#define instr_add(fb, which, a)

/* do this because some compilers may not have long double */
#define get_cpu_speed_measure()			0.0
#define get_cpu_speed_proc()			0.0
#define ticks_to_sec(speed, ticks) 		0.0

#endif /* TC_INSTRUMENT */

/* choose which function you wanna use */
#if 1
#define get_cpu_speed() get_cpu_speed_proc()
#else
#define get_cpu_speed() get_cpu_speed_measure()
#endif

/* timeline */
int tc_time_init();
void tc_time_end();
void tc_time_print();
void tc_time_reset();

#ifdef TC_TIMELINE

/* define this to get a looooot more events reported */
#if 0
#define TC_TIMELINE_EXTRA
#endif

#define gettime_tsc2(tlow, thigh) rdtsc(tlow, thigh)

#define time_time(t) gettime_tsc(t)
#define time_time2(tlow, thigh) gettime_tsc2(tlow, thigh)

#define TC_INSTR_STRLEN	32

/* describes an event */
struct t_irecord {
	char 		str[TC_INSTR_STRLEN];
	unsigned int	ev;
	unsigned long	tlow;
	unsigned long	thigh;
};

/* some predefined events */
enum {
	TI_START
	, TI_END
	, TI_DRAW_START
	, TI_DRAW_END
	, TI_READ_START
	, TI_READ_END
	, TI_NET_START
	, TI_NET_END
} t_time_evs;

/* how many records we have */
#define TC_INSTR_HOWMANY		(1UL<<20)

#define t_str(p)	#p

#define time_print_ev(F, ev)	\
	time_print(F, "%d: %s\n", ev, t_str(ev))

#define time_print_rec(F, i)	\
	time_print(F, "%s %u %lu %lu\n", time_rec_str(i), time_rec_ev(i),\
		time_rec_thigh(i), time_rec_tlow(i))

extern struct t_irecord* t_ievents;
extern unsigned long t_icount;
extern unsigned long t_iwrap;

/* macros */
#define time_inc() 	\
	((++t_icount == TC_INSTR_HOWMANY) && (t_icount = 0) && (++t_iwrap))

#define time_curr()		(t_ievents[t_icount])
#define time_currp()		(&t_ievents[t_icount])
#define time_curr_str()	(time_curr().str)
#define time_curr_ev()		(time_curr().ev)
#define time_curr_tlow()		(time_curr().tlow)
#define time_curr_thigh()		(time_curr().thigh)

#define time_rec(n)		(t_ievents[n])
#define time_recp(n)		(&t_ievents[n])
#define time_rec_str(n)		(time_rec(n).str)
#define time_rec_ev(n)		(time_rec(n).ev)
#define time_rec_tlow(n)		(time_rec(n).tlow)
#define time_rec_thigh(n)		(time_rec(n).thigh)

#define time_strcpy(id) do {				\
	register char *__to = time_curr_str();		\
	register char *__from = (id);			\
	register unsigned int __i = TC_INSTR_STRLEN-1;	\
	while((*__to++ = *__from++) && --__i)		\
		/* nuthin' */ ;				\
	*__to = '\0';					\
} while(0)

#define time_add(_str, _ev) do {			\
	time_strcpy(_str);				\
	time_curr_ev() = (_ev);			\
	time_time2(time_curr_tlow(), time_curr_thigh());	\
	time_inc();					\
} while(0)

/* use this one for events we normally don't want, but may need
 * in some special circumstances */
#ifdef TC_TIMELINE_EXTRA
#define time_add_extra(_str, _ev) time_add(_str, _ev)
#else
#define time_add_extra(_str, _ev)
#endif

#define time_add_time(_str, _ev, _tlow, _thigh) do { 	\
	time_check();					\
	time_strcpy(_str);				\
	time_curr_ev() = (_ev);			\
	time_curr_tlow() = (_tlow);			\
	time_curr_thigh() = (_thigh);			\
	time_inc();					\
} while(0)

/* write something to stdout when we're doing the timeline */
#define time_print(F, ...)	 fprintf(F, __VA_ARGS__)
/* same as time_print but for debugging */
#define time_dbg(s)	tclient_dbg(1, s)

#else					/* TC_TIMELINE */
#define time_time(t)
#define time_time2(tlow, thigh)
#define gettime_tsc2(tlow, thigh)

#define time_print_ev(F, ev)
#define time_print_rec(F, i)

#define time_inc()
#define time_strcpy(id)
#define time_add(id, ev)
#define time_add_time(_id, _ev, _tlow, _thigh)

#define time_curr()	 
#define time_currp()	 
#define time_curr_str() 
#define time_curr_ev()	 
#define time_curr_tlow()
#define time_curr_thigh()

#define time_rec(n)	 
#define time_recp(n)	 
#define time_rec_str(n) 
#define time_rec_ev(n)	 
#define time_rec_tlow(n)
#define time_rec_thigh(n)

#define time_print(...)
#define time_dbg(s)
#define time_add_extra(_str, _ev)

#endif					/* TC_TIMELINE */

#endif /* _TC_INSTRUMENT_H_ */
