/* Copyright 2004, 2005 Ricardo Baratto */
/* Copyright 2004, 2005 Columbia University */
/*
 * Header for event recording
 *
 * Author:
 * Ricardo Baratto
 * Network Computing Lab - Columbia University
 */

/* set tabstop=8 */

#ifndef _TC_RECORD_H_
#define _TC_RECORD_H_

#ifdef TC_RECORD

struct tc_fb;

struct tc_record {
	FILE *fp;	/* where are we saving the events */
	int flags;
	/* for playback */
	unsigned long now;
	char ev;
	/* XXX: Temporary hack */
	struct tc_fb *fb;
};

/* flags */
#define TC_RECORD_MOUSE_CLICK	0x01
#define TC_RECORD_MOUSE_MOVE	0x02
#define TC_RECORD_KEYB		0x04

#define TC_RECORD_ALL	\
	(TC_RECORD_MOUSE_CLICK|TC_RECORD_MOUSE_MOVE|TC_RECORD_KEYB)

/* some defs for the file format, which is:
 * [event] [time] [details of event] */
/* which event this is */
#define TC_REC_MCLICK	'C'
#define TC_REC_MMOVE	'M'
#define TC_REC_KEYB	'K'

/******** record ********/
int tc_record_init(struct tc_record *record, const char *file, int flags);
void tc_record_end(struct tc_record *record);
/* format: [event] [time] [which button] [up or down] */
void tc_record_mouse_click(struct tc_record *record, int which, int updown);
/* format: [event] [time] [x] [y] */
void tc_record_mouse_move(struct tc_record *record, int x, int y);
/* format: [event] [time] [key] [up or down] */
void tc_record_keyb(struct tc_record *record, int key, int updown);

/********** playback ********/
int tc_playback_init(struct tc_fb *fb, const char *file, int which);
int tc_playback_start(struct tc_fb *fb);
void tc_playback_next(struct tc_fb *fb);
void tc_playback_end(struct tc_fb *fb);

void tc_playback_mouse_click(struct tc_fb *fb);
void tc_playback_mouse_move(struct tc_fb *fb);
void tc_playback_keyb(struct tc_fb *fb);


/* macro to convert from ms to microseconds, and fill in the
 * passed tv struct */
#define ms_to_us_tv(ms, tv) \
	((tv).tv_sec = (ms)/1000, (tv).tv_usec = ((ms)%1000)*1000)

#define ms_to_us(ms) 	(ms)*1000

#endif	/* ifdef TC_RECORD */

#endif /* _TC_RECORD_H_ */
